@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: keymap.c,v 1.21 2003/04/29 16:52:04 nalin Exp $"
+#ident "$Id: keymap.c,v 1.24 2003/05/19 22:48:57 nalin Exp $"
 #include "../config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -27,19 +27,15 @@
 #include "caps.h"
 #include "debug.h"
 #include "keymap.h"
-#include "termcap.h"
+#include "vtetc.h"
 #ifdef HAVE_NCURSES
 #include <ncurses.h>
-#ifdef HAVE_TERM_H
 #include <term.h>
-#endif
 #define HAVE_CURSES
 #else
 #ifdef HAVE_CURSES
 #include <curses.h>
-#ifdef HAVE_TERM_H
 #include <term.h>
-#endif
 #else
 #ifdef HAVE_TERMCAP
 #include <termcap.h>
@@ -62,7 +58,7 @@ _vte_keysym_name(guint keyval)
 
 enum _vte_cursor_mode {
 	cursor_default =	1 << 0,
-	cursor_app =		1 << 1,
+	cursor_app =		1 << 1
 };
 
 enum _vte_keypad_mode {
@@ -75,7 +71,7 @@ enum _vte_fkey_mode {
 	fkey_sun =	1 << 1,
 	fkey_hp =	1 << 2,
 	fkey_legacy =	1 << 3,
-	fkey_vt220 =	1 << 4,
+	fkey_vt220 =	1 << 4
 };
 
 #define cursor_all	(cursor_default | cursor_app)
@@ -440,7 +436,7 @@ static struct _vte_keymap_entry _vte_keymap_GDK_KP_Page_Down[] = {
 	 0, _VTE_CAP_CSI "6~", -1, NULL},
 	{cursor_all, keypad_default, fkey_sun, 0, _VTE_CAP_CSI "6z", -1, NULL},
 	{cursor_all, keypad_default, fkey_vt220, 0, "3", 1, NULL},
-	{cursor_all, keypad_app, fkey_notvt220, 0, _VTE_CAP_CSI "5~", -1, NULL},
+	{cursor_all, keypad_app, fkey_notvt220, 0, _VTE_CAP_CSI "6~", -1, NULL},
 	{cursor_all, keypad_app, fkey_vt220, 0, _VTE_CAP_SS3 "s", -1, NULL},
 	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
 };
