@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: nativeecho.c,v 1.2 2003/01/14 03:04:30 nalin Exp $"
+#ident "$Id: nativeecho.c,v 1.3 2003/05/01 20:50:53 nalin Exp $"
 #include "../config.h"
 #include <limits.h>
 #include <stdio.h>
@@ -40,10 +40,10 @@ main(int argc, char **argv)
 
 	for (i = 1; i < argc; i++) {
 		l = strtol(argv[i], &p, 0);
-		while (l > 0) {
+		do {
 			printf("%c", (unsigned char) (l & 0xff));
 			l = l >> 8;
-		}
+		} while (l > 0);
 	}
 
 	return 0;
