@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: pty.c,v 1.37 2002/12/10 22:36:29 nalin Exp $"
+#ident "$Id: pty.c,v 1.45 2003/06/03 19:54:15 jrb Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -44,7 +44,9 @@
 #include "debug.h"
 #include "pty.h"
 
+#ifdef VTE_USE_GNOME_PTY_HELPER
 #include "../gnome-pty-helper/gnome-pty.h"
+#endif
 
 #ifdef MSG_NOSIGNAL
 #define PTY_RECVMSG_FLAGS MSG_NOSIGNAL
@@ -59,10 +61,12 @@
 #define _(String) String
 #endif
 
+#ifdef VTE_USE_GNOME_PTY_HELPER
 static gboolean _vte_pty_helper_started = FALSE;
 static pid_t _vte_pty_helper_pid = -1;
 static int _vte_pty_helper_tunnel = -1;
 static GTree *_vte_pty_helper_map = NULL;
+#endif
 
 /* Reset the handlers for all known signals to their defaults.  The parent
  * (or one of the libraries it links to) may have changed one to be ignored. */
@@ -197,6 +201,72 @@ _vte_pty_pipe_open_bi(int *a, int *b, int *c, int *d)
 	return ret;
 }
 
+/* Like read, but hide EINTR and EAGAIN. */
+static ssize_t
+n_read(int fd, void *buffer, size_t count)
+{
+	size_t n = 0;
+	char *buf = buffer;
+	int i;
+	while (n < count) {
+		i = read(fd, buf + n, count - n);
+		switch (i) {
+		case 0:
+			return n;
+			break;
+		case -1:
+			switch (errno) {
+			case EINTR:
+			case EAGAIN:
+#ifdef ERESTART
+			case ERESTART:
+#endif
+				break;
+			default:
+				return -1;
+			}
+			break;
+		default:
+			n += i;
+			break;
+		}
+	}
+	return n;
+}
+
+/* Like write, but hide EINTR and EAGAIN. */
+static ssize_t
+n_write(int fd, const void *buffer, size_t count)
+{
+	size_t n = 0;
+	const char *buf = buffer;
+	int i;
+	while (n < count) {
+		i = write(fd, buf + n, count - n);
+		switch (i) {
+		case 0:
+			return n;
+			break;
+		case -1:
+			switch (errno) {
+			case EINTR:
+			case EAGAIN:
+#ifdef ERESTART
+			case ERESTART:
+#endif
+				break;
+			default:
+				return -1;
+			}
+			break;
+		default:
+			n += i;
+			break;
+		}
+	}
+	return n;
+}
+
 /* Run the given command, using the given descriptor as the controlling
  * terminal. */
 static int
@@ -279,9 +349,9 @@ _vte_pty_run_on_pty(int fd, int ready_reader, int ready_writer,
 		fprintf(stderr, "Child sending child-ready.\n");
 	}
 #endif
-	write(ready_writer, &c, 1);
+	n_write(ready_writer, &c, 1);
 	fsync(ready_writer);
-	read(ready_reader, &c, 1);
+	n_read(ready_reader, &c, 1);
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_PTY)) {
 		fprintf(stderr, "Child received parent-ready.\n");
@@ -354,7 +424,7 @@ _vte_pty_fork_on_pty_name(const char *path, int parent_fd, char **env_add,
 			fprintf(stderr, "Parent ready, waiting for child.\n");
 		}
 #endif
-		read(ready_a[0], &c, 1);
+		n_read(ready_a[0], &c, 1);
 #ifdef VTE_DEBUG
 		if (_vte_debug_on(VTE_DEBUG_PTY)) {
 			fprintf(stderr, "Parent received child-ready.\n");
@@ -366,7 +436,7 @@ _vte_pty_fork_on_pty_name(const char *path, int parent_fd, char **env_add,
 			fprintf(stderr, "Parent sending parent-ready.\n");
 		}
 #endif
-		write(ready_b[1], &c, 1);
+		n_write(ready_b[1], &c, 1);
 		close(ready_a[0]);
 		close(ready_b[1]);
 
@@ -447,7 +517,7 @@ _vte_pty_fork_on_pty_fd(int fd, char **env_add,
 			fprintf(stderr, "Parent ready.\n");
 		}
 #endif
-		read(ready_a[0], &c, 1);
+		n_read(ready_a[0], &c, 1);
 #ifdef VTE_DEBUG
 		if (_vte_debug_on(VTE_DEBUG_PTY)) {
 			fprintf(stderr, "Parent received child-ready.\n");
@@ -459,7 +529,7 @@ _vte_pty_fork_on_pty_fd(int fd, char **env_add,
 			fprintf(stderr, "Parent sending parent-ready.\n");
 		}
 #endif
-		write(ready_b[1], &c, 1);
+		n_write(ready_b[1], &c, 1);
 		close(ready_a[0]);
 		close(ready_b[1]);
 
@@ -585,16 +655,29 @@ static char *
 _vte_pty_ptsname(int master)
 {
 #if defined(HAVE_PTSNAME_R)
-	char buf[PATH_MAX];
-	memset(buf, 0, sizeof(buf));
-	if (ptsname_r(master, buf, sizeof(buf) - 1) == 0) {
+	gsize len = 1024;
+	char *buf = NULL;
+	int i;
+	do {
+		buf = g_malloc0(len);
+		i = ptsname_r(master, buf, len - 1);
+		switch (i) {
+		case 0:
+			/* Return the allocated buffer with the name in it. */
 #ifdef VTE_DEBUG
-		if (_vte_debug_on(VTE_DEBUG_PTY)) {
-			fprintf(stderr, "PTY slave is `%s'.\n", buf);
-		}
+			if (_vte_debug_on(VTE_DEBUG_PTY)) {
+				fprintf(stderr, "PTY slave is `%s'.\n", buf);
+			}
 #endif
-		return g_strdup(buf);
-	}
+			return buf;
+			break;
+		default:
+			g_free(buf);
+			buf = NULL;
+			break;
+		}
+		len *= 2;
+	} while ((i != 0) && (errno == ERANGE));
 #elif defined(HAVE_PTSNAME)
 	char *p;
 	if ((p = ptsname(master)) != NULL) {
@@ -629,6 +712,9 @@ _vte_pty_getpt(void)
 #else
 	/* Try to allocate a pty by accessing the pty master multiplex. */
 	fd = open("/dev/ptmx", O_RDWR | O_NOCTTY);
+	if ((fd == -1) && (errno == ENOENT)) {
+		fd = open("/dev/ptc", O_RDWR | O_NOCTTY); /* AIX */
+	}
 #endif
 	/* Set it to blocking. */
 	flags = fcntl(fd, F_GETFL);
@@ -702,7 +788,7 @@ _vte_pty_open_unix98(pid_t *child, char **env_add,
 	return fd;
 }
 
-#ifdef HAVE_SENDMSG
+#ifdef HAVE_RECVMSG
 static void
 _vte_pty_read_ptypair(int tunnel, int *parentfd, int *childfd)
 {
@@ -763,6 +849,7 @@ _vte_pty_read_ptypair(int tunnel, int *parentfd, int *childfd)
 #endif
 #endif
 
+#ifdef VTE_USE_GNOME_PTY_HELPER
 static void
 _vte_pty_stop_helper(void)
 {
@@ -789,7 +876,11 @@ _vte_pty_start_helper(void)
 	int i, tmp[2], tunnel;
 	/* Sanity check. */
 	if (access(LIBEXECDIR "/gnome-pty-helper", X_OK) != 0) {
-	 	return FALSE;
+		/* Give the user some clue as to why session logging is not
+		 * going to work (assuming we can open a pty using some other
+		 * method). */
+		g_warning(_("can not run %s"), LIBEXECDIR "/gnome-pty-helper");
+		return FALSE;
 	}
 	/* Create a communication link for use with the helper. */
 	tmp[0] = open("/dev/null", O_RDONLY);
@@ -853,23 +944,43 @@ _vte_pty_open_with_helper(pid_t *child, char **env_add,
 	if (_vte_pty_helper_started) {
 		ops = op;
 		/* Send our request. */
-		if (write(_vte_pty_helper_tunnel,
-			  &ops, sizeof(ops)) != sizeof(ops)) {
+		if (n_write(_vte_pty_helper_tunnel,
+			    &ops, sizeof(ops)) != sizeof(ops)) {
 			return -1;
 		}
+#ifdef VTE_DEBUG
+		if (_vte_debug_on(VTE_DEBUG_PTY)) {
+			fprintf(stderr, "Sent request to helper.\n");
+		}
+#endif
 		/* Read back the response. */
-		if (read(_vte_pty_helper_tunnel,
-			 &ret, sizeof(ret)) != sizeof(ret)) {
+		if (n_read(_vte_pty_helper_tunnel,
+			   &ret, sizeof(ret)) != sizeof(ret)) {
 			return -1;
 		}
+#ifdef VTE_DEBUG
+		if (_vte_debug_on(VTE_DEBUG_PTY)) {
+			fprintf(stderr, "Received response from helper.\n");
+		}
+#endif
 		if (ret == 0) {
 			return -1;
 		}
+#ifdef VTE_DEBUG
+		if (_vte_debug_on(VTE_DEBUG_PTY)) {
+			fprintf(stderr, "Helper returns success.\n");
+		}
+#endif
 		/* Read back a tag. */
-		if (read(_vte_pty_helper_tunnel,
-			 &tag, sizeof(tag)) != sizeof(tag)) {
+		if (n_read(_vte_pty_helper_tunnel,
+			   &tag, sizeof(tag)) != sizeof(tag)) {
 			return -1;
 		}
+#ifdef VTE_DEBUG
+		if (_vte_debug_on(VTE_DEBUG_PTY)) {
+			fprintf(stderr, "Tag = %p.\n", tag);
+		}
+#endif
 		/* Receive the master and slave ptys. */
 		_vte_pty_read_ptypair(_vte_pty_helper_tunnel,
 				      &parentfd, &childfd);
@@ -879,10 +990,8 @@ _vte_pty_open_with_helper(pid_t *child, char **env_add,
 			close(childfd);
 			return -1;
 		}
-
 #ifdef VTE_DEBUG
-		if (_vte_debug_on(VTE_DEBUG_MISC) ||
-		    _vte_debug_on(VTE_DEBUG_PTY)) {
+		if (_vte_debug_on(VTE_DEBUG_PTY)) {
 			fprintf(stderr, "Got master pty %d and slave pty %d.\n",
 				parentfd, childfd);
 		}
@@ -905,6 +1014,7 @@ _vte_pty_open_with_helper(pid_t *child, char **env_add,
 	}
 	return -1;
 }
+#endif
 
 /**
  * _vte_pty_open:
@@ -934,6 +1044,7 @@ _vte_pty_open(pid_t *child, char **env_add,
 	      gboolean lastlog, gboolean utmp, gboolean wtmp)
 {
 	int ret = -1;
+#ifdef VTE_USE_GNOME_PTY_HELPER
 	int op = 0;
 	int opmap[8] = {
 		GNOME_PTY_OPEN_NO_DB_UPDATE,		/* 0 0 0 */
@@ -961,6 +1072,7 @@ _vte_pty_open(pid_t *child, char **env_add,
 						directory,
 						columns, rows, opmap[op]);
 	}
+#endif
 	if (ret == -1) {
 		ret = _vte_pty_open_unix98(child, env_add, command, argv,
 					   directory, columns, rows);
@@ -983,6 +1095,7 @@ _vte_pty_open(pid_t *child, char **env_add,
 void
 _vte_pty_close(int pty)
 {
+#ifdef VTE_USE_GNOME_PTY_HELPER
 	gpointer tag;
 	GnomePtyOps ops;
 	if (_vte_pty_helper_map != NULL) {
@@ -992,12 +1105,12 @@ _vte_pty_close(int pty)
 			ops = GNOME_PTY_CLOSE_PTY;
 			tag = g_tree_lookup(_vte_pty_helper_map,
 					    GINT_TO_POINTER(pty));
-			if (write(_vte_pty_helper_tunnel,
-				  &ops, sizeof(ops)) != sizeof(ops)) {
+			if (n_write(_vte_pty_helper_tunnel,
+				    &ops, sizeof(ops)) != sizeof(ops)) {
 				return;
 			}
-			if (write(_vte_pty_helper_tunnel,
-				  &tag, sizeof(tag)) != sizeof(tag)) {
+			if (n_write(_vte_pty_helper_tunnel,
+				    &tag, sizeof(tag)) != sizeof(tag)) {
 				return;
 			}
 			/* Remove the item from the map. */
@@ -1005,6 +1118,7 @@ _vte_pty_close(int pty)
 				      GINT_TO_POINTER(pty));
 		}
 	}
+#endif
 }
 
 #ifdef PTY_MAIN
@@ -1020,7 +1134,7 @@ sigchld_handler(int signum)
 int
 main(int argc, char **argv)
 {
-	pid_t child;
+	pid_t child = 0;
 	char c;
 	int ret;
 	signal(SIGCHLD, sigchld_handler);
@@ -1030,17 +1144,17 @@ main(int argc, char **argv)
 			   (argc > 1) ? argv + 1 : NULL,
 			   NULL,
 			   0, 0,
-			   FALSE, FALSE, FALSE);
+			   TRUE, TRUE, TRUE);
 	g_print("Child pid is %d.\n", (int)child);
 	do {
-		ret = read(fd, &c, 1);
+		ret = n_read(fd, &c, 1);
 		if (ret == 0) {
 			break;
 		}
 		if ((ret == -1) && (errno != EAGAIN) && (errno != EINTR)) {
 			break;
 		}
-		write(STDOUT_FILENO, &c, 1);
+		n_write(STDOUT_FILENO, &c, 1);
 	} while (TRUE);
 	return 0;
 }
