@@ -18,7 +18,7 @@
 
 /* The interfaces in this file are subject to change at any time. */
 
-#ident "$Id: vtedraw.c,v 1.13 2003/05/13 04:27:39 nalin Exp $"
+#ident "$Id: vtedraw.c,v 1.15 2003/06/16 21:15:10 nalin Exp $"
 
 #include "../config.h"
 
@@ -129,6 +129,9 @@ GdkColormap *
 _vte_draw_get_colormap(struct _vte_draw *draw, gboolean maybe_use_default)
 {
 	GdkColormap *colormap;
+#if GTK_CHECK_VERSION(2,2,0)
+	GdkScreen *screen;
+#endif
 	g_return_val_if_fail(draw->impl != NULL, NULL);
 	g_return_val_if_fail(draw->impl->get_colormap != NULL, NULL);
 	colormap = draw->impl->get_colormap(draw);
@@ -139,7 +142,12 @@ _vte_draw_get_colormap(struct _vte_draw *draw, gboolean maybe_use_default)
 		return NULL;
 	}
 #if GTK_CHECK_VERSION(2,2,0)
-	colormap = gdk_screen_get_default_colormap(gdk_screen_get_default());
+	if (gtk_widget_has_screen(draw->widget)) {
+		screen = gtk_widget_get_screen(draw->widget);
+	} else {
+		screen = gdk_display_get_default_screen(gtk_widget_get_display(draw->widget));
+	}
+	colormap = gdk_screen_get_default_colormap(screen);
 #else
 	colormap = gdk_colormap_get_system();
 #endif
@@ -238,6 +246,14 @@ _vte_draw_get_text_ascent(struct _vte_draw *draw)
 	return draw->impl->get_text_ascent(draw);
 }
 
+int
+_vte_draw_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	g_return_val_if_fail(draw->impl != NULL, 1);
+	g_return_val_if_fail(draw->impl->get_char_width != NULL, 1);
+	return draw->impl->get_char_width(draw, c, columns);
+}
+
 gboolean
 _vte_draw_get_using_fontconfig(struct _vte_draw *draw)
 {
@@ -257,6 +273,20 @@ _vte_draw_text(struct _vte_draw *draw,
 	draw->impl->draw_text(draw, requests, n_requests, color, alpha);
 }
 
+gboolean
+_vte_draw_char(struct _vte_draw *draw,
+	       struct _vte_draw_text_request *request,
+	       GdkColor *color, guchar alpha)
+{
+	g_return_val_if_fail(draw->started == TRUE, FALSE);
+	g_return_val_if_fail(draw->impl != NULL, FALSE);
+	if (draw->impl->draw_char == NULL) {
+		draw->impl->draw_text(draw, request, 1, color, alpha);
+		return TRUE;
+	}
+	return draw->impl->draw_char(draw, request, color, alpha);
+}
+
 void
 _vte_draw_fill_rectangle(struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
