@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtegl.c,v 1.13 2003/06/01 22:29:20 nalin Exp $"
+#ident "$Id: vtegl.c,v 1.14 2003/06/16 21:15:10 nalin Exp $"
 
 #include "../config.h"
 
@@ -428,6 +428,22 @@ _vte_gl_get_text_ascent(struct _vte_draw *draw)
 	return data->cache->ascent;
 }
 
+static int
+_vte_gl_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	struct _vte_gl_data *data;
+	const struct _vte_glyph *glyph;
+
+	data = (struct _vte_gl_data*) draw->impl_data;
+
+	glyph = _vte_glyph_get(data->cache, c);
+	if (glyph != NULL) {
+		return glyph->width;
+	}
+
+	return _vte_gl_get_text_width(draw) * columns;
+}
+
 static gboolean
 _vte_gl_get_using_fontconfig(struct _vte_draw *draw)
 {
@@ -603,6 +619,7 @@ struct _vte_draw_impl _vte_draw_gl = {
 	_vte_gl_get_text_width,
 	_vte_gl_get_text_height,
 	_vte_gl_get_text_ascent,
+	_vte_gl_get_char_width,
 	_vte_gl_get_using_fontconfig,
 	_vte_gl_draw_text,
 	_vte_gl_draw_char,
