@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtepangox.c,v 1.8 2003/05/06 07:38:39 nalin Exp $"
+#ident "$Id: vtepangox.c,v 1.13 2003/06/16 21:15:10 nalin Exp $"
 
 #include "../config.h"
 
@@ -199,11 +199,11 @@ _vte_pango_x_set_background_color(struct _vte_draw *draw, GdkColor *color)
 
 static void
 _vte_pango_x_set_background_image(struct _vte_draw *draw,
-			          enum VteBgSourceType type,
-			          GdkPixbuf *pixbuf,
-			          const char *file,
-			          const GdkColor *color,
-			          double saturation)
+				  enum VteBgSourceType type,
+				  GdkPixbuf *pixbuf,
+				  const char *file,
+				  const GdkColor *color,
+				  double saturation)
 {
 	GdkPixmap *pixmap;
 	struct _vte_pango_x_data *data;
@@ -211,7 +211,7 @@ _vte_pango_x_set_background_image(struct _vte_draw *draw,
 	data = (struct _vte_pango_x_data*) draw->impl_data;
 	pixmap = vte_bg_get_pixmap(vte_bg_get(), type, pixbuf, file,
 				   color, saturation,
-				   _vte_draw_get_colormap(draw));
+				   _vte_draw_get_colormap(draw, TRUE));
 	if (data->pixmap) {
 		g_object_unref(G_OBJECT(data->pixmap));
 	}
@@ -369,6 +369,12 @@ _vte_pango_x_get_text_ascent(struct _vte_draw *draw)
 	return draw->ascent;
 }
 
+static int
+_vte_pango_x_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	return _vte_pango_x_get_text_width(draw) * columns;
+}
+
 static gboolean
 _vte_pango_x_get_using_fontconfig(struct _vte_draw *draw)
 {
@@ -410,6 +416,15 @@ _vte_pango_x_draw_text(struct _vte_draw *draw,
 	}
 }
 
+static gboolean
+_vte_pango_x_draw_char(struct _vte_draw *draw,
+		       struct _vte_draw_text_request *request,
+		       GdkColor *color, guchar alpha)
+{
+	_vte_pango_x_draw_text(draw, request, 1, color, alpha);
+	return TRUE;
+}
+
 static void
 _vte_pango_x_draw_rectangle(struct _vte_draw *draw,
 			  gint x, gint y, gint width, gint height,
@@ -466,13 +481,16 @@ struct _vte_draw_impl _vte_draw_pango_x = {
 	_vte_pango_x_end,
 	_vte_pango_x_set_background_color,
 	_vte_pango_x_set_background_image,
+	FALSE,
 	_vte_pango_x_clear,
 	_vte_pango_x_set_text_font,
 	_vte_pango_x_get_text_width,
 	_vte_pango_x_get_text_height,
 	_vte_pango_x_get_text_ascent,
+	_vte_pango_x_get_char_width,
 	_vte_pango_x_get_using_fontconfig,
 	_vte_pango_x_draw_text,
+	_vte_pango_x_draw_char,
 	_vte_pango_x_draw_rectangle,
 	_vte_pango_x_fill_rectangle,
 	_vte_pango_x_set_scroll,
