@@ -16,13 +16,16 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vterdb.c,v 1.4 2003/05/28 23:45:09 nalin Exp $"
+#ident "$Id: vterdb.c,v 1.6 2003/06/04 19:04:36 nalin Exp $"
 
 #include "../config.h"
 
 #include <limits.h>
 #include <string.h>
 #include <gtk/gtk.h>
+#ifndef X_DISPLAY_MISSING
+#include <gdk/gdkx.h>
+#endif
 #include "vterdb.h"
 
 #define DEFAULT_ANTIALIAS	TRUE
@@ -76,6 +79,13 @@ _vte_rdb_get(GtkWidget *widget, gboolean screen_setting)
 	gdk_property_get(root, atom, GDK_TARGET_STRING, 0, INT_MAX, FALSE,
 			 &prop_type, NULL, &prop_length,
 			 (guchar**) &prop_data);
+#if GTK_CHECK_VERSION(2,2,0)
+	gdk_display_sync(display);
+#else
+#ifndef X_DISPLAY_MISSING
+	XSync(GDK_DISPLAY(), FALSE);
+#endif
+#endif
 	gdk_error_trap_pop();
 
 	/* Only parse the information if we got a string. */
